-- -*- coding: utf-8 -*-
--
-- Adobe Photoshop Lightroom export plugin for Zenfolio
-- Copyright 2008 Jeffrey Friedl
--
-- http://regex.info/blog/photo-tech/lightroom-zenfolio/
-- 
local PluginVersion      = '20080714.50'
local PluginContactName  = 'Jeffrey Friedl'
local PluginContactEmail = 'jfriedl@yahoo.com'
local PluginUrl          = 'http://regex.info/blog/photo-tech/lightroom-zenfolio/'

--[[----------------------------------------------------------------------------

Version History:

20080714.50 -- Minor tweaks

20080704.49 -- Oops, was sending the same text for both title and caption... now fixed. Really. (I think.)

20080613.47 -- A couple of small bug fixes that might help this run in the 2.0 beta a bit better.

20080607.46 -- Oops, cleaning up one loose end with the debug-log move

20080606.45 -- <p>Minor tweak: Moved the plugin debug log to the Documents / My Documents folder.</p>

               <p><b>Mac Users</b>: if you get an error when you try to install the plugin,
               something along the lines of "this plugin is already installed or not
               compatible with your version of Lightroom", please see the <a
               href='http://regex.info/blog/2007-11-17/638'>install instructions</a> and
               <b>manually remove</b> the previously-installed version(s). If that still
               doesn't let you install the new version by clicking, then manually install
               it. I have no idea why this is happening, so if someone figures it out,
               please let me know.</b>

20080523.44 -- Grafting in the caption-management stuff that I've been
               working on for the 2.0 versions of the plugins.
               <a href='http://regex.info/blog/2008-05-23/822'>See description here</a>.
               
20080411.43 -- Fix for the "_update_status" bug that those few who export non-sRGB JPGs ran into.

20080404.42 -- Some small cleanup to allow the plugin to work with the 2.0 beta

20080315.41 -- Sigh, I'd left some in-progress development stuff in the release I pushed yesterday, sorry.

20080314.40 -- Minor update to allow the plugin to work with <a href='http://regex.info/blog/2008-03-14/762'>Lightroom 1.4</a>.

20080221.39 -- Minor UI tweaks for better appearance on a Mac.
               <br/>Updated Swedish translation.


20080217.38 -- small fix to the wording in the Title/Caption dialog

20080217.37 -- <ul><li>New French translation.</li>
               <li>Now includes a Swedish translation by <a href='http://nicke.sjostrom.fi/'>Niklas Sjöström</a>.</li>
               <li>Can now sort the upload gallery list (alphabetically, most-recently-created first, most-recently-used first), and can use "select at export time" (useful for building export presets).</li>
               <li>Fixed (I hope) the &8220;User is not an owner of an object&#8221; error some have reported.</li>
               <li>Renamed &#8220;Extra tags&#8221; to &#8220;Extra keywords&#8221; to reflect the Zenfolio vernacular.</li>
               </ul>

20080128.36 -- Tidied up <a
               href='http://regex.info/blog/photo-tech/anycommand/#comment-13764'>some
               mistakes in the translation hooks</a>.
               Interested in doing the translations for my plugins for your favorite language?
               It's a difficult and thankless job, but you earn good karma for giving to the greater community.
               If interested, please contact me (<b>jfriedl</b>&#8205;&#64;&#8205;<b>yahoo.com</b>).

20080123.35 -- Fixed another error introduced in .33 related to visiting the destination gallery after upload

20080123.34 -- Fix "internal error" bug introduced in previous build, that affected those upgrading from several versions  back.

20080122.33 -- Added <a href="http://regex.info/blog/photo-tech/anycommand/">"Run Any Command" Piglet</a>.
               <br/>Some internal refactoring to better support the <a href='http://regex.info/blog/2008-01-15/692'>piglet infrastructure</a>.
               <br/>Added tiff option back because I think I figured out why it wasn't letting me upload tiff before.

20080117.32 -- Zenfolio doesn't seem to allow tiff uploads, so I've removed that option.

20080117.31 -- Have export dialog allow file types that any enabled piglets allow. Bail if the resulting filename
               has an extension that can't be uploaded.


20080115.30 -- Added the <a href='http://regex.info/blog/photo-tech/piglets/'>piglet manager</a> for fourth-party add-ons

20080109.29 -- New and tidied French translation

20080107.28 -- Housekeeping changes to allow auto-install on OSX

20080104.27 -- Using a preset with a destination gallery set to "ask at export" no longer limits the selections to
               those that existed at the time the preset was created. Due to my coding bug, the gallery-refresh I intended
               never happened; it does now. Thanks to Ben Pharr for reporting the error.

20071227.26 -- Updated French translation

20071222.25 -- Ensure that a long login status message does not crowd out the logout button.
               Work properly even when someone logs in with an email address instead of a username.

20071217.23 -- Update to the title/caption logic.
               Properly handle a no-gallery situation.
               New French translation file.

20071213.22 -- fix bug introduced in .19

20071213.21 -- oops, removing some debugging code I left in by accident

20071213.20 -- new French translation file

20071213.19 -- Can now filter the export so that only images previously exported to Zenfolio will be processed,
               or, conversely, only images that have not been previously exported to Zenfolio.

20071212.18 -- Minor internationalization prep.

20071210.17 -- More internationalization prep. Added a first draft of a French translation, courtesy of
               <a href='http://www.utiliser-lightroom.fr'>Gilles Theophile</a>.
               (Contact him with any suggested changes.)

20071205.16 -- Can now export the original master file.
               Can now note an image resolution in pixels per inch/cm in the image metadata.

20071204.15 -- More internationalization prep.

20071204.14 -- Fixed keywording (I hope)
               Lots of internal work to lay the groundwork for internationalizing the plugin

20071127.11 -- Send along a per-install random identifier with the version check,
               so I can count how many of each plugin is being used.

20071127.10 -- Don't double-upload images when a 'FormatMessageW' error is trapped

20071127.9 --  Allow for any non-blank login name (it didn't allow email addresses before)

20071127.8 --  Fix typos introduced in previous version

20071126.7 --  <p>Added some HTTP error trapping in an effort to get around the 'FormatMessageW failed' error that
               some Windows users have been seeing:</p>

                <img class='screenshot' src='http://regex.info/i/lr/Trapping-Dialog.gif'/>

               <p>When the trapping is enabled, those errors are ignored, which may allow things to mostly work,
               or may simply lead to silent failure. Features that rely on
               the result of the upload (the new photoid) will stop
               working for uploads that experience a trapped error, such as
               the 'delete previously uploaded versions' feature.</p>

               <p>This may not help, but it is worth a try until the underlying issue can be found and resolved.</p>

20071125.6 --  Now handles properly ampersands in Zenfolio galleries/groups

20071125.5 --  Added a "Save to Local Disk" section to the dialog:
               <img class='screenshot' src='http://regex.info/i/lr/ExportPlugin-SaveLocal.gif'/>

20071123.3 --  Added plugin debugging option, to track HTTP calls

20071123.2 --  Report under the title-selection area the result of what is chosen
               <br/>Added 'extra tags' to the upload-destination section (renamed "extra keywords" as of 20080217.37)
               <br/>Added ability to choose where the caption is taken from.

20071121.1 --  initial public release


--]]----------------------------------------------------------------------------

-- Lightroom Export-API imports
local LrFunctionContext = import 'LrFunctionContext'   
local LrProgressScope   = import 'LrProgressScope'
local LrPathUtils       = import 'LrPathUtils'
local LrDialogs         = import 'LrDialogs'
local LrBinding         = import 'LrBinding'
local LrColor           = import 'LrColor'
local LrPrefs           = import 'LrPrefs'
local LrView            = import 'LrView'
local LrHttp            = require 'LrHttpDebug' { logging = false, trapping = false}
local LrXml             = import 'LrXml'

uLOC = LOC -- this is needed for my automated build tools

--require 'jfriedlDebug.lua' -- defines Dump() and DebugNote()

-- local plugin imports
local translator_info         = require 'translator_info'
local colorspace_check        = require 'colorspace_check'(LOC("$$$/175=Note: Regardless, ^1 converts non-sRGB images to sRGB for display.", "Zenfolio"))
local caption_source          = require 'caption_source' { provider = 'Zenfolio', sets = {{ tag = 'title',   label = uLOC("$$$/Zenfolio/Title=title"),   p1 = 'T', p2 = 'H', p3 = 'F', p4 = 'X' },
                                                                                          { tag = 'caption', label = uLOC("$$$/Zenfolio/Caption=caption"), p1 = 'C', p2 = 'X', p3 = 'X', p4 = 'X' }} }
local version_check           = require 'version_check'('zenfolio', PluginVersion, PluginUrl)
local upload_filter           = require 'upload_filter'({ ID = 'photoIds', provider = 'Zenfolio' })
local save_to_disk            = require 'save_to_disk' { provider = 'Zenfolio' }
local async_task              = require 'async_task'
local resolution              = require 'resolution'
local table_util              = require 'table-util'
local Piglets                 = require 'piglet_manager.lua'
local util                    = require 'util'
local interactive_destination = require 'interactive_destination'({
                                                                     provider         = 'Zenfolio',
                                                                     list_key         = 'zenfolio_galleries',
                                                                     select_key       = 'zenfolio_destination_url',
                                                                     title_target     = LOC("$$$/178=Zenfolio Gallery"),
                                                                     text_target      = LOC("$$$/179=destination gallery")
                                                                  })



plugin = { }
function plugin.captions_config()
   return {
      { key = 'title_SPEC',   label = uLOC("$$$/Zenfolio/Title=Title"),     spec = Caption.BASIC_TITLE    },
      { key = 'caption_SPEC', label = uLOC("$$$/Zenfolio/Caption=Caption"), spec = Caption.BASIC_CAPTION  },
   }
end
require 'CaptionStuff.lua'


--
-- A few basic constants we'll need
--
local ZenfolioApiHttpUrl    =  'http://www.zenfolio.com/zf/api/zfapi.asmx'
local ZenfolioApiHttpsUrl   = 'https://www.zenfolio.com/zf/api/zfapi.asmx'
local ZenfolioApiUploadBase =  'http://www.zenfolio.com'

local CollectionOfLightroomExports = LOC("$$$/180=Exported from Lightroom")
local CollectionOfPreviousExport   = LOC("$$$/181=Previous Lightroom Export")

local ZenfolioApiUserAgent = 'Adobe Photoshop Lightroom Export Plugin (Version ' .. PluginVersion .. '; ' .. PluginContactEmail .. ')'

--
-- Return basic HTTP headers required for any Zenfolio API access
--
local function BasicRequestHeaders()
   return {
      { field = "User-Agent",              value = ZenfolioApiUserAgent    },
      { field = "X-Zenfolio-User-Agent",   value = ZenfolioApiUserAgent    },
   }
end

--
-- Uses the username/password from state and tries to get an authentication key.
--
local function AuthenticateCredentials(state, credentials)

   local PreviousStatus = state._login_status_message
   state._login_status_message = LOC("$$$/182=Authenticating credentials at ^1^.", "Zenfolio")

   -- If we're going to try to authenticate, ensure that we blow away any previous token
   state._auth_token = nil

   --
   -- Build the authentication request.
   --
   local xml = LrXml.createXmlBuilder(true)
   if credentials then
      xml:tag('loginName', credentials.username)
      xml:tag('password',  credentials.password)
   else
      xml:tag('loginName', state.zenfolio_username)
      xml:tag('password',  state.zenfolio_password)
   end
   local CredentialsXml = xml:serialize()

   local RequestBody = [[
<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
  <soap:Body>
    <AuthenticatePlain xmlns="http://www.zenfolio.com/api">
      ]] .. CredentialsXml .. [[
    </AuthenticatePlain>
  </soap:Body>
</soap:Envelope>]]

   --
   -- Build the HTTP headers for the request
   --
   local RequestHeaders = BasicRequestHeaders()
   table.insert(RequestHeaders, { field = "Content-Type", value = 'text/xml; charset=utf-8' })
   table.insert(RequestHeaders, { field = "SOAPAction",   value = 'http://www.zenfolio.com/api/AuthenticatePlain' })

   --
   -- Go fetch it
   --
   local ResultBody = LrHttp.post(ZenfolioApiHttpsUrl, RequestBody, RequestHeaders)

   --
   -- Look for a token and an error.
   -- If we get neither, there must be some connection problem
   --
   state._auth_token = ResultBody:match('<AuthenticatePlainResult>([^<]+)<')
   local Error = ResultBody:match('<faultstring>([^<]+)<')

   if ResultBody == "TRAPPED ERROR" then
      Error = "TRAPPED ERROR"
   end

   state._login_status_message = PreviousStatus
   return Error
end


--
-- Clear out the state to a "not logged in" situation
--
local function Logout(state)
   state._auth_token = nil
   state.zenfolio_password = ''
   state.zenfolio_display_name = "unknown" -- this is never used while not logged in, but best to set it "just in case"
   state.zenfolio_login_name = nil
   state.zenfolio_galleries = nil
   state.zenfolio_groups = nil
   state.zenfolio_url_to_name = nil
   state._zenfolio_allexport_collection_id  = nil
   state._zenfolio_prevexport_collection_id = nil
   state._tmp_password = nil
end


--
-- Return the HTTP headers needed for an authenticated request.
--
local function AuthenticatedRequestHeaders(state)

   -- start out with the basic headers
   local Headers = BasicRequestHeaders()

   -- go authenticate if not already authenticated
   if not state._auth_token then
      AuthenticateCredentials(state)
   end

   --
   -- If we're still not authenticated, the username/password pair must no
   -- longer be valid, so force a logout and return nothing
   --
   if not state._auth_token then
      state._login_status_message = LOC("$$$/183=Can't log in: invalid credentials")
      LrDialogs.message(LOC("$$$/184=Can't log in to ^1", "Zenfolio"),
                        LOC("$$$/185=Username and/or Password are no longer valid"),
                        "critical")
      Logout(state)
      return nil
   end

   -- add the authentication token to the header
   table.insert(Headers, { field = "X-Zenfolio-Token", value = state._auth_token })

   -- and return to the user
   return Headers
end


local function ExecuteWithAutoReauthenticate(state, options, func, url, ...)

   -- default options are empty
   options = options or { }

   local ResultBody, ResultHeaders = func(url, ...)

   if ResultHeaders and ResultHeaders.status == 403 then

       -- our authentication key is old... get a new one
      AuthenticateCredentials(state)

      if not state._auth_token or state._auth_token == '' then
         -- error has already been reported, so just return
         return nil
      end

      -- find the headers in our argument list
      local RequestHeaders
      if func == LrHttp.get then
         -- for LrHttp.get, it's the first remaining header
         RequestHeaders = ...
      elseif func == LrHttp.postMultipart then
         -- for postMultipart, it's the second remaining header
         local dummy_mime_chunks
         dummy_mime_chunks, RequestHeaders = ...
      else
         LrDialogs.message(LOC("$$$/101=INTERNAL PLUGIN ERROR"),
                           LOC("$$$/186=Not prepared for function [^1]", tostring(func)))
         assert(nil)
      end

      -- insert new token into the proper field
      for num, header in pairs(RequestHeaders) do
         if header.field == "X-Zenfolio-Token" then
            header.value = state._auth_token
            break
         end
      end

      -- now try a 2nd time, and return
      ResultBody, ResultHeaders = func(url, ...)
   end

   if not ResultBody or (ResultBody == '' and not options.emptyOkay) then
      LrDialogs.message(LOC("$$$/187=Couldn't connect to ^1", "Zenfolio"),
                        LOC("$$$/188=Internet connection down?"),
                        "critical")
      return nil, ResultHeaders
   else
      return ResultBody, ResultHeaders
   end
end



--
-- Very ugly way to grok some XML. This should really use xpath or something sane.
--
--
-- THIS MUST NOT BE 'local' BECAUSE IT CALLS ITSELF.
--
function GrokHeirarchyXml(state, prefix, node)

   local galleries = { }
   local groups    = { }

   local GroupTitle = LOC("$$$/189=HOME PAGE")
   local GroupId    = nil
   --local LightroomAllCollectionId = nil

   for i = 1, node:childCount() do

      local child = node:childAtIndex(i)

      if child:name() == 'Group' then

         local sub_galleries, sub_groups = GrokHeirarchyXml(state, prefix, child)

         -- append the results we got from that level to our own
         for i,v in ipairs(sub_galleries) do table.insert(galleries, v) end
         for i,v in ipairs(sub_groups)    do table.insert(groups,    v) end

      elseif child:name() == 'Title' then

         GroupTitle = prefix .. child:childAtIndex(1):serialize()
         prefix = GroupTitle .. " > "

      elseif child:name() == 'Id' then

         GroupId = child:childAtIndex(1):serialize()

      elseif child:name() == 'Elements' then

         local sub_galleries, sub_groups = GrokHeirarchyXml(state, prefix, child)

         -- append the results we got from that level to our own
         for i,v in ipairs(sub_galleries) do table.insert(galleries, v) end
         for i,v in ipairs(sub_groups)    do table.insert(groups,    v) end

         --if all_id then
         --   LightroomAllCollectionId = all_id
         --end
         --if prev_id then
         --   LightroomPrevCollectionId = prev_id
         --end

      elseif child:name() == 'PhotoSet' then

         local Title        = nil
         local UploadUrl    = nil
         local Id           = nil
         local CreateDate   = nil
         local ModDate = nil

         for j = 1, child:childCount() do
            local subChild =  child:childAtIndex(j)
            if subChild:name() == 'Title' then
               Title = subChild:childAtIndex(1):serialize()
            elseif subChild:name() == 'UploadUrl' then
               UploadUrl = subChild:childAtIndex(1):serialize()
            elseif subChild:name() == 'Id' then
               Id = subChild:childAtIndex(1):serialize()
            elseif subChild:name() == 'CreatedOn' then
               CreateDate = subChild:childAtIndex(1):serialize()
            elseif subChild:name() == 'ModifiedOn' then
               ModDate = subChild:childAtIndex(1):serialize()
            end
         end

         if Title then 
            if UploadUrl then
               table.insert(galleries, { title      = prefix ..  Title,
                                         value      = UploadUrl,
                                         CreateDate = CreateDate,
                                         ModDate    = ModDate })
            else

               -- A PhotoSet without an upload url is a collection
               if Title == CollectionOfLightroomExports and Id then
                  state._zenfolio_allexport_collection_id  = Id
               elseif Title == CollectionOfPreviousExport and Id then
                  state._zenfolio_prevexport_collection_id = Id
               end

            end
         end
      end
   end

   if GroupTitle and GroupId then
      table.insert(groups, 1, { title = GroupTitle, value = GroupId })
   end

   return galleries, groups
end

local function sort_gallery_list(state)

   local tmp = state.zenfolio_galleries

   table.sort(tmp, function(a,b)
                      -- Items that begin with "--" always go at the end of the list
                      if a.title:match('^%-%-') then
                         if b.title:match('^%-%-') then
                            return string.lower(a.title) < string.lower(b.title)
                         else
                            return false
                         end
                      end
                      if b.title:match('^%-%-') then
                         return true
                      end

                      if state.zenfolio_gallery_sort == 'Title' then
                         return string.lower(a.title) < string.lower(b.title)
                      elseif state.zenfolio_gallery_sort == 'ModTime' then
                         return a.ModDate > b.ModDate
                      else
                         return a.CreateDate > b.CreateDate
                      end
                   end)

   state.zenfolio_galleries = tmp -- so that LrBind notices the change
end



--
-- Get the list of user Groups/Galleries and save to state. The "login_name"
-- arg is optional, and defaults to state.zenfolio_login_name
--
local function GetZenfolioGroupInfo(state, login_name)

   local RequestHeaders = AuthenticatedRequestHeaders(state)
   if not RequestHeaders then
      return false
   end

   --
   -- Keep the user informed about what we're doing
   --
   local PreviousStatus = state._login_status_message
   state._login_status_message = LOC("$$$/190=Fetching gallery list from ^1^.", "Zenfolio")

   if not login_name then
      login_name = state.zenfolio_login_name
   end

   local url = ZenfolioApiHttpUrl .. "/LoadGroupHierarchy?loginName=" .. login_name

   --
   -- Fetch the list of galleries
   --
   local ResultBody, ResultHeaders = ExecuteWithAutoReauthenticate(state,
                                                                   nil,
                                                                   LrHttp.get,
                                                                   url,
                                                                   RequestHeaders)


   if not ResultBody then
      return false -- error already reported in ExecuteWithAutoReauthenticate
   end

   if not ResultHeaders or ResultHeaders.status ~= 200 then
      state._login_status_message = LOC("$$$/191=Couldn't get gallery list for ^1 from ^2", login_name, "Zenfolio")
      LrDialogs.message(LOC("$$$/192=Failure contacting ^1 for '^2'", "Zenfolio", login_name) .. "\n" .. tostring(ResultBody))
      return false
   end

   --
   -- Grok the PhotoSet info from the XML
   --
   state._zenfolio_allexport_collection_id  = nil
   state._zenfolio_prevexport_collection_id = nil

   local galleries, groups = GrokHeirarchyXml(state, "", LrXml.parseXml(ResultBody))

   interactive_destination.add_select_element(state, galleries)

   util.tidy_popupmenu_items(galleries)
   util.tidy_popupmenu_items(groups)

   state.zenfolio_galleries                = galleries
   state.zenfolio_groups                   = groups
   state.zenfolio_url_to_name              = {}

   sort_gallery_list(state)

   --
   -- If we had a gallery previously selected, we must make sure that it is still
   -- here. If not, or if we didn't have one selected before, we'll pick the first one.
   --
   if #galleries == 0 then
      state.zenfolio_destination_url = ''
      state.zenfolio_destination_url_at_export = ''
   else
      -- populate zenfolio_url_to_name
      for i, set in ipairs(galleries) do
         state.zenfolio_url_to_name[set.value] = set.title
      end

      -- set the destination gallery to the first one if the chosen one is no longer
      -- in there.
      if not util.popupmenu_items_has_value(galleries, state.zenfolio_destination_url) then
         state.zenfolio_destination_url           = galleries[1].value
         state.zenfolio_destination_url_at_export = galleries[1].value
      end
   end


   if #groups == 0 then
      state.zenfolio_basegroup = nil
   elseif not util.popupmenu_items_has_value(groups, state.zenfolio_basegroup) then
      state.zenfolio_basegroup = groups[1].value
   end

   state._login_status_message = PreviousStatus

   return true
end



local function CreateNew(state, arg)
   --
   -- Build the body
   --
   state._creating_new_gallery = true

   local xml = LrXml.createXmlBuilder(true)
   xml:tag('Title', arg.name)
   local GalleryName = xml:serialize()

   local soapCommand
   local type
   local id
   if arg.what == 'Group' then

      id = '<parentId>' .. arg.base .. '</parentId>'
      type = ''
      soapCommand = 'CreateGroup'

   elseif arg.what == 'Collection' then

      type = '<type>Collection</type>'
      id = '<groupId>' .. arg.base .. '</groupId>'
      soapCommand = 'CreatePhotoSet'

   else

      type = '<type>Gallery</type>'
      id = '<groupId>' .. arg.base .. '</groupId>'
      soapCommand = 'CreatePhotoSet'

   end

   local RequestBody = [[
<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
  <soap:Body>
    <]] .. soapCommand .. [[ xmlns="http://www.zenfolio.com/api">
      ]] .. id .. type .. [[
      <updater>
        ]].. GalleryName ..[[
        <Caption></Caption>
      </updater>
    </]].. soapCommand .. [[>
  </soap:Body>
</soap:Envelope>]]

   --
   -- Build the HTTP headers for the request
   --
   local RequestHeaders = AuthenticatedRequestHeaders(state)
   if not RequestHeaders then
      state._creating_new_gallery = nil
      return -- error has already been reported in AuthenticatedRequestHeaders()
   end

   table.insert(RequestHeaders, { field = "Content-Type", value = 'text/xml; charset=utf-8' })
   table.insert(RequestHeaders, { field = "SOAPAction",   value = 'http://www.zenfolio.com/api/' .. soapCommand })

   --
   -- Keep the user informed about what we're doing
   --
   local PreviousStatus = state._login_status_message
   state._login_status_message = LOC("$$$/193=Creating new ^1 ^[^2^] at ^3^.", string.lower(arg.what), arg.name, "Zenfolio")


   --
   -- Go fetch it
   --
   local ResultBody, ResultHeaders = ExecuteWithAutoReauthenticate(state,
                                                                   nil,
                                                                   LrHttp.post,
                                                                   ZenfolioApiHttpUrl,
                                                                   RequestBody,
                                                                   RequestHeaders)

   if not ResultBody then
      state._creating_new_gallery = nil
      return -- error has already been reported in AuthenticatedRequestHeaders()
   end

   local Error = ResultBody:match('<faultstring>([^<]+)<')

   if ResultBody == "TRAPPED ERROR" then
      Error = "TRAPPED ERROR" 
   end

   if Error then

      LrDialogs.message(LOC("$$$/194=Error"), Error)

   elseif arg.what == 'Gallery' then

      local UploadUrl = ResultBody:match('<UploadUrl>([^<]+)<')

      if UploadUrl then
         GetZenfolioGroupInfo(state)
         state.zenfolio_destination_url           = UploadUrl
         state.zenfolio_destination_url_at_export = UploadUrl
      else
         LrDialogs.message(LOC("$$$/195=Unknown Response from ^1", "Zenfolio"), tostring(ResultBody), "critical")
      end

   elseif arg.what == 'Collection' then

      -- nothing extra to do for a Collection

   else
      GetZenfolioGroupInfo(state)
      state.zenfolio_basegroup = ResultBody:match('<Id>%s*(%d+)')

   end
   state._login_status_message = PreviousStatus

   state._creating_new_gallery = nil
   return ResultBody:match('<Id>%s*(%d+)')
end

local function MakePrivate(state, what, id)
   --
   -- Build the HTTP headers for the request
   --
   local RequestHeaders = AuthenticatedRequestHeaders(state)
   if not RequestHeaders then
      return
   end

   local SoapCommand
   local Tag


   if what == 'Group' then
      SoapCommand = 'UpdateGroupAccess'
      IdTag = '<groupId>' .. id .. '</groupId>'
   else
      SoapCommand = 'UpdatePhotoSetAccess'
      IdTag = '<photoSetId>' .. id .. '</photoSetId>'
   end

   table.insert(RequestHeaders, { field = "Content-Type", value = 'text/xml; charset=utf-8' })
   table.insert(RequestHeaders, { field = "SOAPAction",   value = 'http://www.zenfolio.com/api/' .. SoapCommand })

   local RequestBody = [[
<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
  <soap:Body>
    <]] .. SoapCommand .. [[ xmlns="http://www.zenfolio.com/api">
      ]].. IdTag ..[[
      <updater>
        <AccessType>Private</AccessType>
      </updater>
     </]].. SoapCommand ..[[>
  </soap:Body>
 </soap:Envelope>]]

   --
   -- Go fetch it
   --
   local ResultBody, ResultHeaders = ExecuteWithAutoReauthenticate(state,
                                                                   nil,
                                                                   LrHttp.post,
                                                                   ZenfolioApiHttpUrl,
                                                                   RequestBody,
                                                                   RequestHeaders)

   if not ResultBody then
      return -- error already reported in ExecuteWithAutoReauthenticate
   end

   local Error = ResultBody:match('<faultstring>([^<]+)<')

   if ResultBody == "TRAPPED ERROR" then
      Error = "TRAPPED ERROR" 
   end

   if Error then
      LrDialogs.message(LOC("$$$/196=Couldn't make image private"), Error, "critical")
   end

end

local function CreateNewGroupOrGallery(state)
   local arg = {
      name = state._new_gallery_name,
      what = state.zenfolio_what_to_create,
      base = state.zenfolio_basegroup,
   }
   state._new_gallery_name = ''

   local id = CreateNew(state, arg)
   if id and not state.zenfolio_new_gallery_public then
      MakePrivate(state, state.zenfolio_what_to_create, id)
   end
end


local function CreateCollection(state, name, base)
   return CreateNew(state, {
                       name = name,
                       what = 'Collection',
                       base = base or state.zenfolio_groups[1].value,
                    })
end



--
-- Fetches some basic user info from Zenfolio, such as the user name and
-- how large an image they can upload.
--
local function GetZenfolioUserInfo(state)

   local RequestHeaders = AuthenticatedRequestHeaders(state)
   if not RequestHeaders then
      return
   end

   --
   -- Keep the user informed about what we're doing
   --
   local PreviousStatus = state._login_status_message
   state._login_status_message = LOC("$$$/197=Fetching user information from ^1^.", "Zenfolio")


   --
   -- Go fetch the user info (name, etc.)
   --
   local ResultBody, ResultHeaders = ExecuteWithAutoReauthenticate(state,
                                                                   nil,
                                                                   LrHttp.get,
                                                                   ZenfolioApiHttpUrl .. "/LoadPrivateProfile",
                                                                   RequestHeaders)

   if not ResultBody then
      return -- error already reported in ExecuteWithAutoReauthenticate
   end

   if not ResultHeaders or ResultHeaders.status ~= 200 then
      state._login_status_message = LOC("$$$/198=Couldn't get user data from ^1", "Zenfolio")
      LrDialogs.message(LOC("$$$/199=Failure contacting ^1", "Zenfolio") ..  "\n" .. tostring(ResultBody))
      Logout(state)
      return
   end

   state.zenfolio_login_name   = ResultBody:match('<LoginName>([^<]+)<')
   state.zenfolio_display_name = ResultBody:match('<DisplayName>([^<]+)<') or state.zenfolio_login_name

   -- we don't use the maximum image size quota yet, but may in the future...
   --state.zenfolio_photo_bytes_quota = ResultBody:match('<PhotoBytesQuota>([^<]+)<') or nil

   state._login_status_message = PreviousStatus
end


local function Login(state)
   --
   -- Grab a copy of the credentials that we'll try to log in with
   --
   local credentials = { username = state._tmp_username,
                         password = state._tmp_password }

   --
   -- Be sure to clean out everything that we intend to fill in, just in case we die halfway through
   --
   Logout(state)

   local Error = AuthenticateCredentials(state, credentials)

   if Error then
      LrDialogs.message(LOC("$$$/200=The login attempt failed"), Error, "critical")
      Logout(state)

   elseif not state._auth_token then
      -- no error, and no auth token, so must have been some kind of connection issue
      LrDialogs.message(LOC("$$$/187=Couldn't connect to ^1", "Zenfolio"),
                        LOC("$$$/188=Internet connection down?"),
                        "critical")
      state._login_status_message = LOC("$$$/187=Couldn't connect to ^1", "Zenfolio")
   else
      GetZenfolioUserInfo(state)
      GetZenfolioGroupInfo(state)

      -- Now that we've logged in and fetched what we need, we can consider
      -- ourselves logged in, which copying these over does.
      state.zenfolio_username = credentials.username
      state.zenfolio_password = credentials.password
   end
end



local function creategallery_dialog(v, state, context, selection_item)

   if context == 'interactive destination' then
      state._buttons_are_enabled = true
      state._enable_create_gallery_button = false
      --state._refreshing_user_data = nil
      assert(selection_item)

      local STATE = LrPrefs.prefsForPlugin(_PLUGIN.id)
      state:addObserver(selection_item, function()
                                           state.zenfolio_destination_url_at_export = state[selection_item]
                                           STATE.zenfolio_destination_url_at_export = state[selection_item]
                                       end)
   end

   local function get_items_to_use()

      if context ~= 'interactive destination' then

         selection_item = 'zenfolio_destination_url'
         state._items_to_use = state.zenfolio_galleries

      else

         state._items_to_use = interactive_destination.filter_items(state.zenfolio_galleries)

         if state.zenfolio_destination_url_at_export ~= "" and util.popupmenu_items_has_value(state._items_to_use, state.zenfolio_destination_url_at_export) then
            state[selection_item] = state.zenfolio_destination_url_at_export
         elseif state.zenfolio_destination_url ~= "" and util.popupmenu_items_has_value(state._items_to_use, state.zenfolio_destination_url) then
            state[selection_item] = state.zenfolio_destination_url
         else
            state[selection_item] = state._items_to_use[1].value
         end
      end

   end


   get_items_to_use()
   state:addObserver('zenfolio_galleries',                 get_items_to_use)
   state:addObserver('zenfolio_destination_url',           get_items_to_use)
   state:addObserver('zenfolio_destination_url_at_export', get_items_to_use)
   

   state:addObserver('zenfolio_gallery_sort',  function()
                                                 sort_gallery_list(state)
                                                 get_items_to_use()
                                              end)

   --
   -- Enable the create-gallery button when the user starts typing a gallery name.
   --
   local function check_gallery_name()
      state._enable_create_gallery_button = state._new_gallery_name ~= ''
   end
   state:addObserver('_new_gallery_name', check_gallery_name)
   check_gallery_name()


   return v:view {
      bind_to_object = state,

      v:view {
         place = 'overlapping',

         v:static_text {
            visible = LrView.bind('_not_logged_in'),
            title = LOC("$$$/219=Not currently logged in."),
         },

         v:view {
            visible = LrBinding.andAllKeys('_is_logged_in', '_have_some_galleries'),
            spacing = v:label_spacing(),
            margin = util.WinMac(0, 2),

            v:view {
               place = 'horizontal',

               v:static_text {
                  title = LOC("$$$/220=Upload to your ^1 gallery:", "Zenfolio"),
                  width = LrView.share '_x2'
               },

               v:view {
                  place = 'overlapping',
                  margin_bottom = 3,

                  v:static_text {
                     visible = LrBinding.keyIsNotNil '_creating_new_gallery',
                     title = LOC("$$$/426=working^."),
                     text_color = LrColor(1, .2, .1),
                  },

                  v:popup_menu {
                     visible = LrBinding.keyIsNil '_creating_new_gallery',
                     items = LrView.bind('_items_to_use'),
                     value = LrView.bind(selection_item),
                     width_in_chars = tonumber(util.WinMac(LOC("$$$/221/prefix/Win=40"), LOC("$$$/221/prefix/Mac=35"))),
                     tooltip = LOC("$$$/221=This list is refreshed each time you open the Export dialog"),
                  },
               },
            },

            v:view {
               spacing = 0,
               margin = 0,
               margin_bottom = 5,

               place='horizontal',
               v:static_text {
                  title = " ",
                  width = LrView.share '_x2',
               },
               v:radio_button {
                  size = 'mini',
                  title = LOC("$$$/295=sort alphabetically"),
                  value = LrView.bind('zenfolio_gallery_sort'),
                  checked_value = 'Title',
               },
               v:spacer {
                  width = 15,
               },
               v:radio_button {
                  size = 'mini',
                  title = LOC("$$$/429=recently-made first"),
                  value = LrView.bind('zenfolio_gallery_sort'),
                  checked_value = 'Time',
               },
               v:spacer {
                  width = 15,
               },
               v:radio_button {
                  size = 'mini',
                  title = LOC("$$$/430=recently-used first"),
                  value = LrView.bind('zenfolio_gallery_sort'),
                  checked_value = 'ModTime',
               },
            },

            v:view {
               fill_horizontal = 1,
               spacing = 0,
               margin = 0,

               v:row {
                  fill_horizontal = 1,
                  v:static_text { title = LOC("$$$/431=Extra keywords") },
                  v:edit_field  { fill_horizontal = 1,
                                  value = LrView.bind "zenfolio_extra_tags" },
               },

               v:view {
                  spacing = 0,
                  margin = 0,
                  place = 'horizontal',
                  
                  v:static_text {
                     title = "",
                     width_in_chars = tonumber(util.WinMac(LOC("$$$/223/prefix/Win=6"),LOC("$$$/223/prefix/Mac=6")))
                  },

                  v:static_text { title = "(" },

                  v:static_text {
                     title = LOC("$$$/223=comma-separated; enclose in quotes as needed, e.g.:"),
                     font = "<system/small>"
                  },

                  v:static_text { title = LOC("$$$/224=flower,  with vase,  \"rose, red\""),
                                  font = "<system/small/bold>",
                                  text_color = LrColor(.5,.5,1) },

                  v:static_text { title = ")",
                                  font = "<system/small>" }
               },
            },
         },

         v:static_text {
            visible = LrBinding.andAllKeys('_is_logged_in', '_have_no_galleries'),
            title = LOC("$$$/225=You have no galleries at ^1 to upload to.^nYou can create one below.", "Zenfolio"),
            text_color = LrColor(1,0,0),
            font = "<system/bold>",
         },
      },


      v:group_box {
         visible = LrView.bind('_is_logged_in'),
         title = util.WinMac(LOC("$$$/226= create  "),
                             nil),
         place = 'vertical',
         fill_horizontal = 1,

         v:static_text {
            title = LOC("$$$/227=You can use this to add a new group or gallery to your Zenfolio account.^n" ..
                        "A new gallery automatically becomes the new upload destination."),
            tooltip = LOC("$$$/228=A group can hold galleries and groups, while a gallery can hold only images. You upload only to a gallery, but you can choose to use groups to help manage your galleries in a hierarchical fashion."),
         },

         v:view {
            place = 'horizontal',

            v:static_text { title = LOC("$$$/229=Under") },

            v:popup_menu {
               items = LrView.bind('zenfolio_groups'),
               value = LrView.bind('zenfolio_basegroup'),
               width_in_chars = util.WinMac(15,7),
            },

            v:static_text { title = LOC("$$$/230= create") },

            v:popup_menu {
               items = { { title = LOC("$$$/231=public"),  value = true },
                         { title = LOC("$$$/232=private"), value = false } },
               value = LrView.bind('zenfolio_new_gallery_public'),
               width_in_chars = 4,
            },

            v:popup_menu {
               items = { { title = LOC("$$$/233=gallery"), value = 'Gallery' },
                         { title = LOC("$$$/234=group"),   value = 'Group' } },
               value = LrView.bind('zenfolio_what_to_create'),
               width_in_chars = tonumber(util.WinMac(LOC("$$$/233/width/Win=4"),LOC("$$$/233/width/Mac=4"))),
            },

            v:static_text { title = LOC("$$$/235= named:") },

            v:edit_field {
               immediate = true,
               alignment = 'left',
               value = LrView.bind('_new_gallery_name'),
               width_in_chars = util.WinMac(11,11),
               tooltip = LOC("$$$/236=Exporting is disabled while typing into this text input box."),
            },

            v:push_button {
               title = LOC("$$$/237=Create"),
               enabled = LrBinding.andAllKeys('_buttons_are_enabled', '_enable_create_gallery_button'),

               action = function(button)
                           state._enable_create_gallery_button = false
                           async_task.run(state,
                                          LOC("$$$/388=Create new ^1 at Zenfolio", state.zenfolio_what_to_create),
                                          CreateNewGroupOrGallery)
                        end,
            },
         },
      },
   }
end

interactive_destination.args().creategallery_dialog = creategallery_dialog
interactive_destination.args().refresh_function = GetZenfolioGroupInfo
interactive_destination.args().create_gallery = function(state)
                                                   if state._new_gallery_name ~= '' then
                                                      CreateNewGroupOrGallery(state)
                                                   end
                                                end



local function sectionsForTopOfDialog(v, state)

   state._creating_new_gallery = nil

   -- this is just in case things get discombobulated
   if state.zenfolio_username == nil or state.zenfolio_username == '' then
      state.zenfolio_password = ''
      state.zenfolio_login_name = nil
   end

   local function check_credential_input()
      local UsernameIsSyntaticallyValid = state._tmp_username:match('^[^%s]+$') or false
      local PasswordIsSyntaticallyValid = state._tmp_password and #state._tmp_password >= 6
      state._login_button_enabled = UsernameIsSyntaticallyValid and PasswordIsSyntaticallyValid
   end

   function update_status()

      local canExport = true -- we'll assume we can, unless we can't

      if not state.zenfolio_login_name then
         canExport = false
         state._login_status_message = LOC("$$$/138=Not logged in to ^1", "Zenfolio")
         state._is_logged_in = false
         state._not_logged_in = true
         state._destination_synopsis = ''
         state._have_some_galleries = false
         state._have_no_galleries   = false
      else
         if state.zenfolio_display_name == state.zenfolio_login_name then
            state._login_status_message = LOC("$$$/201=Logged in to ^1 as ^2", "Zenfolio", tostring(state.zenfolio_display_name))
         else
            state._login_status_message = LOC("$$$/202=Logged in to ^1 as ^2 (^3)",
                                              "Zenfolio",
                                              tostring(state.zenfolio_display_name),
                                              tostring(state.zenfolio_login_name))
         end
         state._is_logged_in = true
         state._not_logged_in = false

         state._have_some_galleries = state.zenfolio_galleries and #state.zenfolio_galleries > 1
         state._have_no_galleries   = not state._have_some_galleries

         if state.zenfolio_destination_url == '' or state.zenfolio_galleries == nil or #state.zenfolio_galleries <= 1 then
            canExport = false
            state._destination_synopsis = LOC("$$$/203=---  awaiting configuration  ---")
         else
            if state.zenfolio_url_to_name and state.zenfolio_url_to_name[state.zenfolio_destination_url] then
               state._destination_synopsis = state.zenfolio_url_to_name[state.zenfolio_destination_url]
            else
               state._destination_synopsis = ''
            end
         end
      end

      local color_warning = colorspace_check.synopsis(state)
      if color_warning then
         if state._destination_synopsis == '' then
            state._destination_synopsis = color_warning
         else
            state._destination_synopsis = color_warning .. "; " .. state._destination_synopsis
         end
      end

      --
      -- We don't want to allow export if the user is typing a name
      -- into the new-gallery text-input field, because if they hit ENTER
      -- thinking that it will create the gallery, they'll not create the gallery
      -- and instead go directly to export. To avoid that, we remove the ability to export
      -- when text is being entered.
      --
      if not state._buttons_are_enabled then
         canExport = false
      end
      if state._enable_create_gallery_button then
         canExport = false
      end

      if not state.__save_after_can_export then
         canExport = false
      end

      state.LR_canExport = (canExport and Piglets.canExport(state))

   end
   state._update_status = update_status -- throw a pointer here so that it's easily accessible from anywhere


   --
   -- This is a generic flag that disables all buttons when it's set to
   -- false. When true, it allows a button to become enabled if other
   -- button-specific conditions are met.
   --
   state._buttons_are_enabled = true

   state:addObserver('_new_gallery_name', function ()
                                             state._enable_create_gallery_button = state._new_gallery_name ~= ''
                                             update_status()
                                          end)

   state._destination_synopsis = ''
   state._enable_create_gallery_button = false
   state._new_gallery_name = ''

   state:addObserver('_tmp_username', check_credential_input)
   state:addObserver('_tmp_password', check_credential_input)

   state:addObserver('zenfolio_password',          update_status)
   state:addObserver('zenfolio_destination_url',   update_status)
   state:addObserver('_buttons_are_enabled',       update_status)
   state:addObserver('allow_colorspace_warning',   update_status)
   state:addObserver('__save_after_can_export',    update_status)

   Piglets.set_CanExport_Observer(state, update_status)

   state._tmp_username = state.zenfolio_username
   state._tmp_password = state.zenfolio_password

   state._not_logging_in = true

   update_status()

   if state._is_logged_in then
      async_task.run(state, LOC("$$$/386=^1 info refresh", 'Zenfolio'), GetZenfolioGroupInfo)
   end

   local Sections = {
      {
         title    = LOC("$$$/204=Your ^1 Account", "Zenfolio"),
         place    = 'vertical',
         synopsis = LrView.bind { key = '_login_status_message', object = state },

         v:view {
            place = 'overlapping',
            fill_horizontal = 1,

            -- the "while not logged in" view
            v:view {
               visible = LrBinding.andAllKeys('_not_logged_in', '_not_logging_in'),
               place = 'horizontal',
               fill_horizontal = 1,

               v:view {
                  fill_horizontal = 1,

                  v:view {
                     place = 'horizontal',
                     spacing = v:label_spacing(),

                     v:static_text {
                        title = LOC("$$$/422=Email or Username"),
                        alignment = 'right',
                        width = LrView.share "login_label_width",
                     },

                     v:edit_field {
                        immediate = true,
                        alignment = 'left',
                        value = LrView.bind('_tmp_username'),
                        width_in_chars = 20,
                     },
                  },

                  v:view {
                     place = 'horizontal',
                     spacing = v:label_spacing(),

                     v:static_text {
                        title = LOC("$$$/206=Your ^1 password", "Zenfolio"),
                        alignment = 'right',
                        width = LrView.share "login_label_width",
                        tooltip = LOC("$$$/207=Your password is stored in the Lightroom database, so be sure to log out if you don't want it saved on this machine."),
                     },

                     v:password_field {
                        immediate = true,
                        alignment = 'left',
                        value = LrView.bind('_tmp_password'),
                        width_in_chars = 20,
                        tooltip = LOC("$$$/207=Your password is stored in the Lightroom database, so be sure to log out if you don't want it saved on this machine."),
                     },

                     v:push_button {
                        enabled = LrBinding.andAllKeys('_buttons_are_enabled', '_login_button_enabled'),
                        title = LOC("$$$/208=Login"),
                        action = function(button)
                                    state._not_logging_in = false
                                    async_task.run(state, LOC("$$$/387=^1 login", 'Zenfolio'), Login, nil, function(state)
                                                                                                      state._not_logging_in = true
                                                                                                   end)
                                 end,
                        tooltip = LOC("$$$/207=Your password is stored in the Lightroom database, so be sure to log out if you don't want it saved on this machine."),
                     },
                  },
               },

               v:push_button {
                  title = LOC("$$$/209=Visit ^1", "Zenfolio"),
                  action = function(button)
                              LrHttp.openUrlInBrowser("http://www.zenfolio.com/")
                           end,
                  tooltip = LOC("$$$/110=Press to open ^1 in your browser", "http://www.zenfolio.com/"),
               },
            },


            -- the "while logging in..." view...
            v:view {
               visible = LrBinding.negativeOfKey('_not_logging_in'),
               fill_horizontal = 1,
               place = 'vertical',

               v:static_text {
                  title = LOC("$$$/210=Logging in to ^1^.", "Zenfolio"),
               },
               v:static_text {
                  title = LrView.bind('_login_status_message'),
               },
            },



            -- the "while logged in" view
            v:view {
               visible = LrBinding.andAllKeys('_is_logged_in', '_not_logging_in'),
               fill_horizontal = 1,
               margin = util.WinMac(0, 2),
               place = 'horizontal',
               spacing = v:label_spacing(),

               v:static_text {
                  title = LrView.bind('_login_status_message'),
                  height_in_lines = 2,
                  --width_in_chars = util.WinMac(46,37),
               },

               v:push_button {
                  title = LOC("$$$/211=Visit Your ^1", "Zenfolio"),
                  action = function(button)
                              LrHttp.openUrlInBrowser("http://".. state.zenfolio_login_name ..".zenfolio.com/")
                           end,
               },

               v:push_button {
                  enabled = LrView.bind('_buttons_are_enabled'),
                  title = LOC("$$$/212=Logout"),
                  action = function(button)
                              state._buttons_are_enabled = false
                              Logout(state)
                              state._buttons_are_enabled = true
                           end,
               },
            },
         },
      },
   }
   
   Piglets.appendTopSections(Sections, state, v)

   return Sections
end

local function sectionsForBottomOfDialog(v, passed_in_state)

   state = passed_in_state

   -- Handle checking for caption-source changes (this must be called before the management_synopsis stuff
   caption_source.watch(state)

   --
   -- This section is about the synopsis for the "Upload Management" section
   --
   do
      local function set_management_synopsis()
         local notes = {}

         if state._upload_filter_synopsis then
            table.insert(notes, state._upload_filter_synopsis)
         end

         if state._upload_filter_can_do_old then
            if state.zenfolio_delete_prior_uploads  then  table.insert(notes, LOC("$$$/213=prev DELETED"))
                                                    else  table.insert(notes, LOC("$$$/214=prev retained"))  end
         end

         if state.zenfolio_auto_allcollection    then  table.insert(notes, LOC("$$$/215=auto all"))       end
         if state.zenfolio_auto_prevcollection   then  table.insert(notes, LOC("$$$/216=auto prev"))      end
         if state.zenfolio_visit_after_export    then  table.insert(notes, LOC("$$$/217=visit gallery"))  end

         state._upload_management_synopsis = table.concat(notes, ';   ')
      end

      state:addObserver('zenfolio_visit_after_export',   set_management_synopsis)
      state:addObserver('zenfolio_delete_prior_uploads', set_management_synopsis)
      state:addObserver('zenfolio_auto_allcollection',   set_management_synopsis)
      state:addObserver('zenfolio_auto_prevcollection',  set_management_synopsis)
      state:addObserver('_is_logged_in',                 set_management_synopsis)
      state:addObserver('_upload_filter_synopsis',       set_management_synopsis)
      set_management_synopsis()
   end

   local Sections = {
      {
         title = "Zenfolio: " .. LOC("$$$/218=Upload Destination"),
         synopsis = LrView.bind { key = '_destination_synopsis', object = state },
         spacing = 0,

         creategallery_dialog(v, state, 'export dialog'),
         colorspace_check.dialog_section(v, state),
      },


      save_to_disk.dialog_section(v, state),

      --
      -- TITLE and DESCRIPTION section
      --
      {
         title = "Zenfolio: " .. LOC("$$$/238=Title / Caption / Etc."),
         
         v:tab_view {
            value = LrView.bind 'use_new_caption_style',

            v:tab_view_item {
               title = "old style",
               identifier = 'no',

               v:column {
                  v:static_text { title = LOC("$$$/239=Title:"), font = "<system/bold>" },
                  caption_source.dialog_section(v, state, 'title'),
               },
               v:column {
                  v:static_text { title = LOC("$$$/432=Caption:"), font = "<system/bold>" },
                  caption_source.dialog_section(v, state, 'caption'),
               },
            },
            v:tab_view_item {
               title = "new style",
               identifier = 'yes',
               CaptionStuff(v)
            }
         },

         resolution.dialog_section(v, state),

      },





      --
      -- UPLOAD MANAGEMENT
      --
      {
         title = "Zenfolio: " .. LOC("$$$/241=Upload Management"),
         synopsis = LrView.bind { key = '_upload_management_synopsis', object = state },

         upload_filter.dialog_section(v, state),

         v:checkbox {
            enabled = LrView.bind('_upload_filter_can_do_old'),
            title   = LOC("$$$/242=Delete any previously-uploaded versions of the exported images in your ^1 account", "Zenfolio"),
            value   = LrView.bind('zenfolio_delete_prior_uploads'),
            tooltip = LOC("$$$/243=If this option is checked, after uploading an image, any versions of the same image previously uploaded to ^1 (via this plugin) are deleted from ^1.", "Zenfolio"),
         },

         v:checkbox {
            title   = LOC("$$$/244=Add uploads to private ^[^1^] collection", CollectionOfLightroomExports),
            value   = LrView.bind('zenfolio_auto_allcollection'),
            tooltip = LOC("$$$/245=If this option is checked, each image is added to the collection. If the collection doesn't exist, it's created and made private."),
         },

         v:checkbox {
            title   = LOC("$$$/246=Replace uploads to private ^[^1^] collection", CollectionOfPreviousExport),
            value   = LrView.bind('zenfolio_auto_prevcollection'),
            tooltip = LOC("$$$/245=If this option is checked, each image is added to the collection. If the collection doesn't exist, it's created and made private."),
         },

         v:checkbox {
            title   = LOC("$$$/247=Visit destination gallery after upload"),
            value   = LrView.bind('zenfolio_visit_after_export'),
            tooltip = LOC("$$$/248=If this option is checked, the destination gallery opens up in your web browser after the export has completed."),
         },
      }
   }

   local PluginInfoSection = {
      title = LOC("$$$/249=^1 Export Plugin Info", "Zenfolio"),
      synopsis = LrView.bind { key = 'info_synopsis', object = state },
      spacing = 2,

      v:view {
         place = 'horizontal',
         margin_bottom = 0,

         v:static_text {
            title = LOC("$$$/250=^1 Export Plugin for Lightroom", "Zenfolio"),
            font = "<system/bold>",
         },

         v:static_text { title = LOC("$$$/251=version ^1", PluginVersion) },
      },

      v:view {
         margin_left   = 10,
         spacing       = util.WinMac(0,4),
         margin_bottom = util.WinMac(0,2),

         v:static_text {
            title = LOC("$$$/252= by ^1", PluginContactName),
            tooltip = PluginContactEmail,
         },

         v:push_button {
            font = "<system/small>",
            title = "  " .. PluginUrl .. "  ",
            action = function(button)
                        LrHttp.openUrlInBrowser(PluginUrl)
                     end,
         },
      },

      translator_info.dialog_section(v, state),
      version_check.dialog_section(v, state),

      v:view {
         fill_horizontal = 1,
         place = 'horizontal',
         spacing = 0,
         margin_top = 10,
         margin_bottom = 10,
         Piglets.button(v, state),

         v:spacer { fill_horizontal = 1 },
         v:static_text {
            title = LOC("$$$/x1253=Donations are not required, but may be made"),
            text_color = LrColor(.6)
         },
         v:push_button {
            size = 'mini',
            title = LOC("$$$/x1254=here"),
            action = function(button)
                        LrHttp.openUrlInBrowser("https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=jfriedl%40yahoo%2ecom&item_name=Jeffrey%20Friedl%27s%20Lightroom%20Plugin%20Development&item_number=Zenfolio+Plugin&no_shipping=1&cn=Comment&tax=0&currency_code=USD&lc=US&bn=PP%2dDonationsBF&charset=UTF%2d8")
                     end
         }
      },

      v:view {
         v:static_text {
            fill_horizontal = 1,
            title = LOC("$$$/253=These metadata are sent with each image, even if 'Minimize Embedded Metadata' is checked:") .. "\n"
                          .. "        " .. LOC("$$$/171=Title")     .. "\n"
                          .. "        " .. LOC("$$$/167=Caption")   .. "\n"
                          .. "        " .. LOC("$$$/254=Copyright") .. "\n"
                          .. "        " .. LOC("$$$/255=Keywords")  .. "\n"
          },
      },

      LrHttp.dialog_section(state, v),
   }

   Piglets.appendBottomSections(Sections, state, v)
   table.insert(Sections, PluginInfoSection)

   return Sections
end


local function AddPhotoToCollections(state, photoId, ids)

   local RequestHeaders = AuthenticatedRequestHeaders(state)
   if not RequestHeaders then
      return
   end

   for i, Id in ipairs(ids) do
      local url = tostring(ZenfolioApiHttpUrl) .. "/CollectionAddPhoto?collectionId=" .. tostring(Id) .. "&photoId=" .. tostring(photoId)

      local ResultBody, ResultHeaders = LrHttp.get(url, RequestHeaders)

      if not ResultHeaders or ResultHeaders.status ~= 200 then
         LrDialogs.message(LOC("$$$/256=Couldn't add to collection"), ResultBody, "critical")
      end
   end
end

local function SetPhotoData(state, PhotoId, data)

   local xml = LrXml.createXmlBuilder(true)
   if data.title     then xml:tag('Title',     data.title)     end
   if data.caption   then xml:tag('Caption',   data.caption)   end
   if data.copyright then xml:tag('Copyright', data.copyright) end
   local xmlGuts = xml:serialize() or ''

   if data.keywords then
      xml = LrXml.createXmlBuilder(true)
      for i, word in ipairs(data.keywords)  do
         xml:tag('Keyword', word)
      end
      xmlGuts = xmlGuts .. '<Keywords>\n' .. xml:serialize() .. '</Keywords>'
   end


   if #xmlGuts == 0 then
      return
   end

   local RequestBody = [[
<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
  <soap:Body>
    <UpdatePhoto xmlns="http://www.zenfolio.com/api">
      <photoId>]]..PhotoId..[[</photoId>
      <updater>]]..xmlGuts..[[
      </updater>
    </UpdatePhoto>
  </soap:Body>
 </soap:Envelope>]]

   --
   -- Build the HTTP headers for the request
   --
   local RequestHeaders = AuthenticatedRequestHeaders(state)
   table.insert(RequestHeaders, { field = "Content-Type", value = 'text/xml; charset=utf-8' })
   table.insert(RequestHeaders, { field = "SOAPAction",   value = 'http://www.zenfolio.com/api/UpdatePhoto' })

   --
   -- Go fetch it
   --
   local ResultBody, ResultHeaders = ExecuteWithAutoReauthenticate(state,
                                                                   nil,
                                                                   LrHttp.post,
                                                                   ZenfolioApiHttpUrl,
                                                                   RequestBody,
                                                                   RequestHeaders)
   if not ResultBody then
      return -- error already reported in ExecuteWithAutoReauthenticate
   end


   if not ResultHeaders or ResultHeaders.status ~= 200 then
      LrDialogs.message(LOC("$$$/257=Couldn't set image data"), ResultBody, "critical")
   end

end

local function DeletePhoto(state, photoId)

   local RequestHeaders = AuthenticatedRequestHeaders(state)
   if not RequestHeaders then
      return
   end

   local url = ZenfolioApiHttpUrl .. "/DeletePhoto?photoId=" .. photoId

   local ResultBody, ResultHeaders = ExecuteWithAutoReauthenticate(state,
                                                                   { emptyOkay = true },
                                                                   LrHttp.get,
                                                                   url,
                                                                   RequestHeaders)
   if not ResultBody then
      return -- error already reported in ExecuteWithAutoReauthenticate
   end
   
   if not ResultHeaders or ResultHeaders.status ~= 200 then
      LrDialogs.message(LOC("$$$/258=Couldn't delete image with photoId #^1 at Zenfolio", photoId), ResultBody, "critical")
   end
end

local function remove_photos_from_collection(state, CollectionId, IDs, progress)

   local RequestHeaders = AuthenticatedRequestHeaders(state)
   local total2do = #IDs

   -- progress:setCancelable(true)

   for i, PhotoId in ipairs(IDs) do

      if progress:isCanceled() then
         return
      end
      
      progress:setCaption(LOC("$$$/259=Removing ^1 of ^2 prior images", tostring(i), tostring(total2do)))

      local url = ZenfolioApiHttpUrl .."/CollectionRemovePhoto?collectionId=" .. CollectionId .. "&photoId=" .. PhotoId
      local ResultBody, ResultHeaders = ExecuteWithAutoReauthenticate(state,
                                                                      { emptyOkay = true },
                                                                      LrHttp.get,
                                                                      url,
                                                                      RequestHeaders)

      if not ResultBody then
         return -- error already reported in ExecuteWithAutoReauthenticate
      end

      progress:setPortionComplete(i/total2do)
   end
end

local function ClearOutCollection(state, CollectionId)

   local RequestHeaders = AuthenticatedRequestHeaders(state)
   if not RequestHeaders then
      return
   end

   local url = ZenfolioApiHttpUrl .. "/LoadPhotoSet?photosetId=" .. CollectionId
   local ResultBody, ResultHeaders = ExecuteWithAutoReauthenticate(state,
                                                                   nil,
                                                                   LrHttp.get,
                                                                   url,
                                                                   RequestHeaders)
   
   if not ResultBody then
      return -- error has already been reported in AuthenticatedRequestHeaders()
   end


   local IDs = {}
   for PhotoId in ResultBody:gmatch('<Photo>%s*<Id>(%d+)</Id>') do
      table.insert(IDs, PhotoId)
   end

   if #IDs == 0 then
      return
   end

   local label = LOC("$$$/260=Clearing ^[^1^]", CollectionOfPreviousExport)
   LrFunctionContext.postAsyncTaskWithContext(label,
                                              function(context)
                                                 local progress = LrProgressScope { title = label,
                                                                                    functionContext = context
                                                                                 }

                                                 context:addCleanupHandler(function()
                                                                              progress:done()
                                                                           end)
                                                 remove_photos_from_collection(state, CollectionId, IDs, progress)
                                              end)
end

local function abort_message(num, total)
   if num == 0 then
      return LOC("$$$/261=Aborting export; no images successfully uploaded")
   end
   return LOC("$$$/262=Aborting export after ^1 of ^2 images successfully uploaded", tostring(num), tostring(total))
end

local allowed_file_extensions = {
   jpg  = 1, jpeg = 1,
   tif  = 1, tiff = 1,
}


local function do_one_upload(PIGLET_INFO, args)

   local simpleImageName = LrPathUtils.leafName(PIGLET_INFO.image_path)
   local state = PIGLET_INFO.state

   if not PIGLET_INFO.success then
      -- when not successful, the 2nd argument is really an error message
      local ErrorMessage = PIGLET_INFO.image_path
      LrDialogs.message(LOC("$$$/265=Couldn't render ^1", simpleImageName), ErrorMessage, "critical")
      PIGLET_INFO.progressScope:cancel()
      return 'error'
   end

   local mime_type = util.image_mime_type(PIGLET_INFO.image_path)
   if not mime_type then
      LrDialogs.message(LOC("$$$/425=Can't deduce image MIME type; aborting."),
                        PIGLET_INFO.image_path,
                        "critical")
      PIGLET_INFO.progressScope:cancel()
      return 'error'
   end
   if not allowed_file_extensions[string.lower(LrPathUtils.extension(PIGLET_INFO.image_path))] then
      local extension = LrPathUtils.extension(PIGLET_INFO.image_path)
      LrDialogs.message(LOC("$$$/424=^1 does not accept ^[.^2^] files for upload; aborting.", "Zenfolio", extension),
                        PIGLET_INFO.image_path,
                        "critical")
      PIGLET_INFO.progressScope:cancel()
      return 'error'
   end         


   local MimeChunks =
      {
         {
            name = 'file',
            fileName = simpleImageName,
            filePath = PIGLET_INFO.image_path,
            contentType = mime_type,
         }
      }

   --
   -- Actually upload the file
   --
   local ResultBody, ResultHeaders = ExecuteWithAutoReauthenticate(state,
                                                                   nil,
                                                                   LrHttp.postMultipart,
                                                                   args.destUrl,
                                                                   MimeChunks,
                                                                   args.RequestHeaders)
   if not ResultBody then
      PIGLET_INFO.progressScope:cancel()
      return 'error' -- error already reported in ExecuteWithAutoReauthenticate
   end

   if ResultBody == "TRAPPED ERROR" then

      -- oh well, let's pretend it worked and hope for the best

   elseif not ResultHeaders then
      LrDialogs.message(LOC("$$$/101=INTERNAL PLUGIN ERROR"), "upload ResultHeaders is nil", "critical")
      return 'error'

   elseif ResultHeaders.status == 398 then              --398 User storage quota is exceeded
      LrDialogs.message(LOC("$$$/266=Your storage quota at ^1 has been exceeded", "Zenfolio"),
                        abort_message(success_count, total),
                        "critical")
      PIGLET_INFO.progressScope:cancel()
      return 'error'

   elseif ResultHeaders.status == 399 then              --399 The photo file size is greater than allowed by user's subscription plan
      LrDialogs.message(LOC("$$$/267=File size of ^1 exceeds your ^2 subscription plan", simpleImageName, "Zenfolio"),
                        LOC("$$$/268=It was not uploaded."),
                        "warning")
      return 'error'

   elseif ResultHeaders.status == 499 then              --499 The photo file size exceeds maximum value supported
      LrDialogs.message(LOC("$$$/269=File size of ^1 exceeds ^2 per-image upload limit", simpleImageName, "Zenfolio"),
                        LOC("$$$/268=It was not uploaded."),
                        "warning")
      return 'error'

   elseif ResultHeaders.status == 497 then              --497 The photo dimensions are too big
      LrDialogs.message(LOC("$$$/270=Image dimensions exceed ^1 limits", "Zenfolio"),
                        abort_message(success_count, total),
                        "critical")
      PIGLET_INFO.progressScope:cancel()
      return 'error'

   elseif ResultHeaders.status ~= 200 then

      LrDialogs.message(LOC("$$$/101=INTERNAL PLUGIN ERROR"),
                        LOC("$$$/271=Upload for ^1 had status #^2", simpleImageName, tostring(ResultHeaders.status)),
                        "warning")
      return 'error'

   elseif not ResultBody then
      LrDialogs.message(LOC("$$$/101=INTERNAL PLUGIN ERROR"), "ResultBody is nil", "critical")
      return 'error'

   elseif not ResultBody:match('^[0-9]+$') then
      LrDialogs.message(LOC("$$$/101=INTERNAL PLUGIN ERROR"), LOC("$$$/272=Unexpected Result Body") .. "\n" .. ResultBody, "critical")
      return 'error'

   else
      local ZenfolioPhotoId = ResultBody

      --
      -- Add to any of the auto-collections that have been requested
      --
      do
         local ids = {}
         if state.zenfolio_auto_allcollection and state._zenfolio_allexport_collection_id then
            table.insert(ids, state._zenfolio_allexport_collection_id)
         end
         if state.zenfolio_auto_prevcollection and state._zenfolio_prevexport_collection_id then
            table.insert(ids, state._zenfolio_prevexport_collection_id)
         end
         if #ids > 0 then
            AddPhotoToCollections(state, ZenfolioPhotoId, ids)
         end
      end

      PIGLET_INFO.exportContext.exportSession.catalog:withCatalogDo(function()
                                                           local rendition = PIGLET_INFO.rendition
 
                                                           local data = {
                                                              title     = get_caption(state, rendition.photo, simpleImageName, 'title',   caption_source),
                                                              caption   = get_caption(state, rendition.photo, simpleImageName, 'caption', caption_source),
                                                              copyright = rendition.photo:getFormattedMetadata('copyright'),
                                                           }

                                                           local keyword_string = rendition.photo:getFormattedMetadata('keywordTags')
                                                           if keyword_string then
                                                              data.keywords = util.CsvToTable_NoEmpty(keyword_string)
                                                           else
                                                              data.keywords = {}
                                                           end

                                                           if state.zenfolio_extra_tags then
                                                              local extra = util.CsvToTable_NoEmpty(state.zenfolio_extra_tags)
                                                              data.keywords = table_util.union(data.keywords, extra)
                                                           end

                                                           SetPhotoData(state, ZenfolioPhotoId, data)

                                                           rendition.photo:withSettingsForPluginDo(_PLUGIN.id,

                                                                                                   function (photoState)

                                                                                                      if not photoState.photoIds then
                                                                                                         photoState.photoIds = {}
                                                                                                      end

                                                                                                      if state.zenfolio_delete_prior_uploads then
                                                                                                         for i, photoId in ipairs(photoState.photoIds) do
                                                                                                            DeletePhoto(state, photoId)
                                                                                                         end
                                                                                                         photoState.photoIds = {}
                                                                                                      end
                                                                                                      table.insert(photoState.photoIds, ZenfolioPhotoId)

                                                                                                   end)
                                                        end)

   end

   --
   -- We've uploaded the local file, so now we can move it somewhere if
   -- we want, delete it if we want, or just leave it be and it'll be
   -- deleted by LR automatically once the export is done.
   --
   save_to_disk.handleFile(state, PIGLET_INFO.image_path, simpleImageName)

   Piglets.endOfOneRender(PIGLET_INFO)

   return 'okay'
end


local function processRenderedPhotos(functionContext, exportContext)

   if not upload_filter.apply(exportContext) then
      return
   end

   LrHttp.trapped_count = 0

   local state = exportContext.propertyTable
   local total = exportContext.exportSession:countRenditions()

   local PIGLET_INFO = { state         = state,
                         exportContext = exportContext }

   if not Piglets.priorToRender(PIGLET_INFO) then
      return
   end

   local RequestHeaders = AuthenticatedRequestHeaders(state)
   if not RequestHeaders then
      return
   end

   local DEST_URL = interactive_destination.get(state, total)

   if not DEST_URL then
      return
   end

   if not save_to_disk.preRenderStuff(state) then
      return
   end

   local title
   if total == 1 then
      title = LOC("$$$/263=Uploading one image to ^1", "Zenfolio")
   else
      title = LOC("$$$/264=Uploading ^1 images to ^2", tostring(total), "Zenfolio")
   end

   PIGLET_INFO.progressScope = exportContext:configureProgress { title = title }

   --
   -- If we're to automatically add the exported photos to some collections,
   -- make sure the collections exist, creating them (and marking them private) if necessary.
   --
   if state.zenfolio_auto_allcollection or state.zenfolio_auto_prevcollection then
      GetZenfolioGroupInfo(state)

      if state.zenfolio_auto_allcollection and not state._zenfolio_allexport_collection_id then
         local id = CreateCollection(state, CollectionOfLightroomExports)
         if id then
            MakePrivate(state, 'Collection', id)
            state._zenfolio_allexport_collection_id = id
         end
      end

      if state.zenfolio_auto_prevcollection then
         if not state._zenfolio_prevexport_collection_id then
            local id = CreateCollection(state, CollectionOfPreviousExport)
            if id then
               MakePrivate(state, 'Collection', id)
               state._zenfolio_prevexport_collection_id = id
            end
         else
            -- the "previous exports" collection exists, so we need to clear it out
            ClearOutCollection(state, state._zenfolio_prevexport_collection_id)
         end
      end
   end


   local success_count = 0
   local upload_count = 0

   local args = {
      destUrl = ZenfolioApiUploadBase .. DEST_URL,
      RequestHeaders = RequestHeaders,
   }

   for i, rendition in exportContext:renditions { stopIfCanceled = true } do

      PIGLET_INFO.rendition = rendition
      PIGLET_INFO.success, PIGLET_INFO.image_path = rendition:waitForRender()

      local continue = Piglets.startOfOneRender(PIGLET_INFO)
      if not continue then
         break
      end

      if PIGLET_INFO.progressScope:isCanceled() then
         break
      end

      if continue == 'skip this image' then

         -- it's not an error, so consider it a success
         success_count = success_count + 1

      else

         local status = do_one_upload(PIGLET_INFO, args)
         if status == 'okay' then
            upload_count  = upload_count  + 1
            success_count = success_count + 1
         end

      end

      if PIGLET_INFO.progressScope:isCanceled() then
         break
      end
   end

   local errors = total - success_count

   PIGLET_INFO.image_path     = nil
   PIGLET_INFO.success        = nil
   PIGLET_INFO.rendition      = nil
   PIGLET_INFO.attempted      = total
   PIGLET_INFO.exportedOkay   = success_count
   PIGLET_INFO.exportedError  = errors

   if PIGLET_INFO.progressScope:isCanceled() then
      Piglets.exportWasCanceled(PIGLET_INFO)
      return
   end

   if errors == 1 then
      LrDialogs.message(LOC("$$$/273=Upload Failure"), LOC("$$$/274=One image failed to upload"), "warning")
   elseif errors > 1 then
      LrDialogs.message(LOC("$$$/273=Upload Failure"), LOC("$$$/275=^1 images failed to upload", tostring(errors)), "warning")
   end

   LrHttp.IssueWarning()

   save_to_disk.postRenderStuff(state)

   if upload_count > 0 and state.zenfolio_visit_after_export then
      local url = args.destUrl:gsub('/upload[^/]+', '/home.aspx', 1)
      LrHttp.openUrlInBrowser(url)
   end

   Piglets.exportComplete(PIGLET_INFO)
end

--
-- Finally, we return this to Lightroom to inform it about our export service.
--
return {
   --
   -- Zenfolio-specific configuration of the standard Export dialog sections...
   --
   allowFileFormats = Piglets.allowFileFormats { 'JPEG',
                                                 'TIFF',
                                                 'ORIGINAL' },

   hideSections = {
      'exportLocation',
      'postProcessing',
   },

   hidePrintResolution = true,

   --
   -- init
   --
   startDialog = function(state)
                    Piglets.startDialog(state)
                    upload_filter.init(state)
                 end,

   endDialog   = function(state)
                    Piglets.endDialog(state)
                 end,



   --
   -- Branding for the export-destination part of the export dialog
   --
   medium_icon = 'zenfolio-logo.png',
   title_alignment = 'right',
   title_color = LrColor(.12, .34, .56),

   --
   -- Zenfolio-specific parts of the dialog
   --
   sectionsForTopOfDialog    = sectionsForTopOfDialog,
   sectionsForBottomOfDialog = sectionsForBottomOfDialog,

   --
   -- "Upload to Zenfolio" post-render handler
   --
   processRenderedPhotos = processRenderedPhotos,

   --
   -- Data that I want to keep around from invocation to invocation
   --
   exportPresetFields = table_util.union(
    {
       { key = 'zenfolio_username',                  default = ''       }, -- this is misnamed... it's what the user logs in with (email or login_name)
       { key = 'zenfolio_password',                  default = ''       }, -- associated password for the login
       { key = 'zenfolio_login_name',                default = nil      }, -- the "login name" used to create URLs, etc.
       { key = 'zenfolio_display_name',              default = nil      }, -- how we should address the user
       { key = 'zenfolio_galleries',                 default = nil      }, -- table of name/URL mappings of User's galleries
       { key = 'zenfolio_url_to_name',               default = nil      }, -- simple reverse mapping of zenfolio_galleries
       { key = 'zenfolio_groups',                    default = nil      }, -- name/id  mapping of User's groups
       { key = 'zenfolio_destination_url',           default = ''       }, -- user-chosen gallery's upload URL
       { key = 'zenfolio_destination_url_at_export', default = ''       }, 
       { key = 'zenfolio_basegroup',                 default = ''       }, -- user-chosen group in which to create galleries
       { key = 'zenfolio_what_to_create',            default = 'Gallery'}, -- create Group or Gallery?
       { key = 'zenfolio_visit_after_export',        default = true     }, -- if true, visit gallery after export
       { key = 'zenfolio_delete_prior_uploads',      default = false    }, -- if true, delete any previously-uploaded versions of an exported photo
       { key = 'zenfolio_new_gallery_public',        default = true     }, -- if true, newly-created group/gallery is public
       { key = 'zenfolio_extra_tags',                default = ''       }, -- extra keywords to add to each image heading out
       { key = 'zenfolio_gallery_sort',              default = 'Title'  },
       { key = 'zenfolio_auto_allcollection',        default = true     }, -- if true, auto-add all exported images to the zenfolio_allexport_collection_id 
       { key = 'zenfolio_auto_prevcollection',       default = true     }, -- if true, auto-add the exported images to the zenfolio_prevexport_collection_id 
       { key = 'plugin_debugging',                   default = false    },
       { key = 'use_new_caption_style',              default = 'no'     },

    },
    interactive_destination.exportPresetFields(),
    colorspace_check.exportPresetFields(),
    caption_source.exportPresetFields(),
    version_check.exportPresetFields(),
    upload_filter.exportPresetFields(),
    save_to_disk.exportPresetFields(),
    resolution.exportPresetFields(),
    Piglets.exportPresetFields(),
    LrHttp.exportPresetFields(),
    Caption.exportPresetFields()
   )
}
