--
-- Copyright 2008 Jeffrey Friedl
-- http://regex.info/blog/
--
-- 
-- Provides support to my export modules to allow the user to select which metadata
-- is used as the title/caption at the upload destination.
--
local LrView = import 'LrView'

local LongDesc = {
   F = uLOC("$$$/001/long=generated filename"),
   C = uLOC("$$$/002/long=Caption"),
   T = uLOC("$$$/003/long=Title"),
   N = uLOC("$$$/004/long=Copy Name"),
   H = uLOC("$$$/005/long=Headline"),
   J = uLOC("$$$/006/long=Job ID"),
}

local ShortDesc = {
   F = uLOC("$$$/001/short=filename"),
   C = uLOC("$$$/002/short=Caption"),
   T = uLOC("$$$/003/short=Title"),
   N = uLOC("$$$/004/short=CopyName"),
   H = uLOC("$$$/005/short=Headline"),
   J = uLOC("$$$/006/short=Job ID"),
}

local caption_source = {}

local my -- filled in when imported by the plugin

function caption_source.note_key(tag)

   -- default to the first set if no set specified
   if not tag then
      tag = my.sets[1].tag 
   end

   return '_source_priority_note_' .. tag

end

function caption_source.synopsis_key(tag)

   -- default to the first set if no set specified
   if not tag then
      tag = my.sets[1].tag 
   end

   return '_source_priority_synopsis_' .. tag

end


local function __SOURCE_ENABLE(tag, which)
   return '_source_enable_' .. tag .. '_' .. which
end

local function __CAPTION(tag, which)
   return 'caption_' .. tag .. '_' .. which
end


function caption_source.watch(state)

   for i, set in ipairs(my.sets) do
      local tag = set.tag

      local function SOURCE_ENABLE(which)
         return __SOURCE_ENABLE(tag, which)
      end

      local function CAPTION(which)
         return __CAPTION(tag, which)
      end

      state[SOURCE_ENABLE 'c1'] = true
      state[SOURCE_ENABLE 'c2'] = true
      state[SOURCE_ENABLE 'c3'] = true
      state[SOURCE_ENABLE 'c4'] = true

      state[SOURCE_ENABLE 't1'] = true
      state[SOURCE_ENABLE 't2'] = true
      state[SOURCE_ENABLE 't3'] = true
      state[SOURCE_ENABLE 't4'] = true

      state[SOURCE_ENABLE 'f1'] = true
      state[SOURCE_ENABLE 'f2'] = true
      state[SOURCE_ENABLE 'f3'] = true
      state[SOURCE_ENABLE 'f4'] = true

      state[SOURCE_ENABLE 'h1'] = true
      state[SOURCE_ENABLE 'h2'] = true
      state[SOURCE_ENABLE 'h3'] = true
      state[SOURCE_ENABLE 'h4'] = true

      state[SOURCE_ENABLE 'n1'] = true
      state[SOURCE_ENABLE 'n2'] = true
      state[SOURCE_ENABLE 'n3'] = true
      state[SOURCE_ENABLE 'n4'] = true

      state[SOURCE_ENABLE 'j1'] = true
      state[SOURCE_ENABLE 'j2'] = true
      state[SOURCE_ENABLE 'j3'] = true
      state[SOURCE_ENABLE 'j4'] = true

      local function check_source_order()
         --
         -- you can't have something selected in two priority slots...
         --
         if state[CAPTION 'p1'] ~= 'X' then
            if state[CAPTION 'p1'] == state[CAPTION 'p2'] then
               state[CAPTION 'p2'] = 'X'
            end
            if state[CAPTION 'p1'] == state[CAPTION 'p3'] then
               state[CAPTION 'p3'] = 'X'
            end
            if state[CAPTION 'p1'] == state[CAPTION 'p4'] then
               state[CAPTION 'p4'] = 'X'
            end
         end
         if state[CAPTION 'p2'] ~= 'X' then
            if state[CAPTION 'p2'] == state[CAPTION 'p3'] then
               state[CAPTION 'p3'] = 'X'
            end
            if state[CAPTION 'p2'] == state[CAPTION 'p4'] then
               state[CAPTION 'p4'] = 'X'
            end
         end
         if state[CAPTION 'p3'] ~= 'X' and  state[CAPTION 'p3'] == state[CAPTION 'p4'] then
            state[CAPTION 'p4'] = 'X'
         end

         --
         -- If an earlier priority is switched to 'unused', shift down the
         -- other priorities to fill
         --
         if state[CAPTION 'p3'] == 'X' then
            state[CAPTION 'p3'] = state[CAPTION 'p4']
            state[CAPTION 'p4'] = 'X'
         end
         if state[CAPTION 'p2'] == 'X' then
            state[CAPTION 'p2'] = state[CAPTION 'p3']
            state[CAPTION 'p3'] = state[CAPTION 'p4']
            state[CAPTION 'p4'] = 'X'
         end
         if state[CAPTION 'p1'] == 'X' then
            state[CAPTION 'p1'] = state[CAPTION 'p2']
            state[CAPTION 'p2'] = state[CAPTION 'p3']
            state[CAPTION 'p3'] = state[CAPTION 'p4']
            state[CAPTION 'p4'] = 'X'
         end

         --
         -- If Filename or 'unused' are selected at a priority level, any
         -- lower-level priorities must be 'unused'. This is true for 'unused'
         -- so that we front-fill the prioriites, and is true for 'Filename' because
         -- every image has a filename, so it makes no sense to have anything lower
         -- in the priority stream.
         --
         if state[CAPTION 'p1'] == 'F' or state[CAPTION 'p1'] == 'X' then
            state[CAPTION 'p2'] = 'X'
            state[CAPTION 'p3'] = 'X'
            state[CAPTION 'p4'] = 'X'
         end

         if state[CAPTION 'p2'] == 'F' or state[CAPTION 'p2'] == 'X' then
            state[CAPTION 'p3'] = 'X'
            state[CAPTION 'p4'] = 'X'
         end

         if state[CAPTION 'p3'] == 'F' or state[CAPTION 'p3'] == 'X' then
            state[CAPTION 'p4'] = 'X'
         end

         --
         -- If something is selected at one priority level, it can't be selected at the other priority levels
         --
         state[SOURCE_ENABLE 'c2'] = ( state[CAPTION 'p1'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'C' )


         state[SOURCE_ENABLE 'c3'] = ( state[CAPTION 'p1'] ~= 'F' and state[CAPTION 'p2'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'C' and state[CAPTION 'p2'] ~= 'C' )


         state[SOURCE_ENABLE 'c4'] = ( state[CAPTION 'p1'] ~= 'F' and state[CAPTION 'p2'] ~= 'F' and state[CAPTION 'p3'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'C' and state[CAPTION 'p2'] ~= 'C' and state[CAPTION 'p3'] ~= 'C' )


         state[SOURCE_ENABLE 't2'] = ( state[CAPTION 'p1'] ~= 'F' and
                                        state[CAPTION 'p1'] ~= 'T' )


         state[SOURCE_ENABLE 't3'] = ( state[CAPTION 'p1'] ~= 'F' and state[CAPTION 'p2'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'T' and state[CAPTION 'p2'] ~= 'T' )

         state[SOURCE_ENABLE 't4'] = ( state[CAPTION 'p1'] ~= 'F' and state[CAPTION 'p2'] ~= 'F' and state[CAPTION 'p3'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'T' and state[CAPTION 'p2'] ~= 'T' and state[CAPTION 'p3'] ~= 'T' )


         state[SOURCE_ENABLE 'h2'] = ( state[CAPTION 'p1'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'H' )


         state[SOURCE_ENABLE 'h3'] = ( state[CAPTION 'p1'] ~= 'F' and state[CAPTION 'p2'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'H' and state[CAPTION 'p2'] ~= 'H' )

         state[SOURCE_ENABLE 'h4'] = ( state[CAPTION 'p1'] ~= 'F' and state[CAPTION 'p2'] ~= 'F' and state[CAPTION 'p3'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'H' and state[CAPTION 'p2'] ~= 'H' and state[CAPTION 'p3'] ~= 'H' )

         state[SOURCE_ENABLE 'n2'] = ( state[CAPTION 'p1'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'N' )

         state[SOURCE_ENABLE 'n3'] = ( state[CAPTION 'p1'] ~= 'F' and state[CAPTION 'p2'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'N' and state[CAPTION 'p2'] ~= 'N' )

         state[SOURCE_ENABLE 'n4'] = ( state[CAPTION 'p1'] ~= 'F' and state[CAPTION 'p2'] ~= 'F' and state[CAPTION 'p3'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'N' and state[CAPTION 'p2'] ~= 'N' and state[CAPTION 'p3'] ~= 'N' )


         state[SOURCE_ENABLE 'j2'] = ( state[CAPTION 'p1'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'J')

         state[SOURCE_ENABLE 'j3'] = ( state[CAPTION 'p1'] ~= 'F' and state[CAPTION 'p2'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'J' and state[CAPTION 'p2'] ~= 'J' )

         state[SOURCE_ENABLE 'j4'] = ( state[CAPTION 'p1'] ~= 'F' and state[CAPTION 'p2'] ~= 'F' and state[CAPTION 'p3'] ~= 'F'
                                        and
                                        state[CAPTION 'p1'] ~= 'J' and state[CAPTION 'p2'] ~= 'J' and state[CAPTION 'p3'] ~= 'J' )


         state[SOURCE_ENABLE 'f2'] = ( state[CAPTION 'p1'] ~= 'F' )
         state[SOURCE_ENABLE 'f3'] = ( state[CAPTION 'p1'] ~= 'F'  and state[CAPTION 'p2'] ~= 'F' )
         state[SOURCE_ENABLE 'f4'] = ( state[CAPTION 'p1'] ~= 'F'  and state[CAPTION 'p2'] ~= 'F' and state[CAPTION 'p3'] ~= 'F')


         if state[CAPTION 'p1'] == 'X' then

            state[caption_source.note_key(tag)] = LOC("$$$/155/new=No ^2 will be explicitly sent, but ^1 will process the image metadata in its own way.",
                                                      my.provider, set.label)
            state[caption_source.synopsis_key(tag)] = LOC("$$$/156=no ^1", set.label)

         elseif state[CAPTION 'p2'] == 'X' then

            state[caption_source.note_key(tag)] = LOC("$$$/157=The ^1 will be used for each ^2.", LongDesc[state[CAPTION 'p1']], set.label)
            state[caption_source.synopsis_key(tag)] = ShortDesc[state[CAPTION 'p1']]

         elseif state[CAPTION 'p3'] == 'X' then

            state[caption_source.note_key(tag)] = LOC("$$$/158=The ^1 will be used for each ^2 if it exists, otherwise the ^3 will be used.",
                                                      LongDesc[state[CAPTION 'p1']],
                                                      set.label,
                                                      LongDesc[state[CAPTION 'p2']])

            state[caption_source.synopsis_key(tag)] = ShortDesc[state[CAPTION 'p1']] .. '|' .. ShortDesc[state[CAPTION 'p2']]

         elseif state[CAPTION 'p4'] == 'X' then

            state[caption_source.synopsis_key(tag)] = ShortDesc[state[CAPTION 'p1']] .. '|' .. ShortDesc[state[CAPTION 'p2']] .. '|' .. ShortDesc[state[CAPTION 'p3']]

            state[caption_source.note_key(tag)] = LOC("$$$/159=The first available from among ^1, ^2, and ^3 will be used for each ^4.",
                                                      LongDesc[state[CAPTION 'p1']],
                                                      LongDesc[state[CAPTION 'p2']],
                                                      LongDesc[state[CAPTION 'p3']],
                                                      set.label)
         else
            state[caption_source.synopsis_key(tag)] = ShortDesc[state[CAPTION 'p1']] .. '|' .. ShortDesc[state[CAPTION 'p2']] .. '|' .. ShortDesc[state[CAPTION 'p3']] .. '|' .. ShortDesc[state[CAPTION 'p4']]
            state[caption_source.note_key(tag)] = LOC("$$$/160=The first available from among ^1, ^2, ^3, and ^4^nwill be used for each ^5.",
                                                      LongDesc[state[CAPTION 'p1']],
                                                      LongDesc[state[CAPTION 'p2']],
                                                      LongDesc[state[CAPTION 'p3']],
                                                      LongDesc[state[CAPTION 'p4']],
                                                      set.label)
         end
      end

      state:addObserver(CAPTION('p1'),   check_source_order)
      state:addObserver(CAPTION('p2'),   check_source_order)
      state:addObserver(CAPTION('p3'),   check_source_order)
      state:addObserver(CAPTION('p4'),   check_source_order)

      check_source_order()
   end
end


function caption_source.exportPresetFields()

   local items = {}

   for i, set in ipairs(my.sets) do
      table.insert(items, { key = __CAPTION(set.tag, 'p1'), default = set.p1 })
      table.insert(items, { key = __CAPTION(set.tag, 'p2'), default = set.p2 })
      table.insert(items, { key = __CAPTION(set.tag, 'p3'), default = set.p3 })
      table.insert(items, { key = __CAPTION(set.tag, 'p4'), default = set.p4 })
   end

   return items
end

function caption_source.dialog_section(v, state, tag)

   -- default to the first set if no set specified
   if not tag then
      tag = my.sets[1].tag 
   end

   local set = my.tag2set[tag]
   assert(set)

   local function SOURCE_ENABLE(which)
      return __SOURCE_ENABLE(tag, which)
   end

   local function CAPTION(which)
      return __CAPTION(tag, which)
   end

   return v:view {
      fill_horizontal = 1,

      v:static_text { title = LOC("$$$/161=Select priority for the Lightroom metadata to be used as the image ^1 at ^2:",
                                  set.label,
                                  my.provider) },

      v:column {
         margin_top = 5,
         margin_left = 50,
         margin_bottom = 5,

         v:row {
            v:static_text { title = LOC("$$$/162=  SOURCE            priority:"),      width = LrView.share "_w1"   },
            v:static_text { title = LOC("$$$/163=1st"),   width = LrView.share "_w2"  },
            v:static_text { title = LOC("$$$/164=2nd"),   width = LrView.share "_w2"  },
            v:static_text { title = LOC("$$$/165=3rd"),   width = LrView.share "_w2"  },
            v:static_text { title = LOC("$$$/166=4th"),   width = LrView.share "_w2"  },
         },

         v:separator { fill_horizontal = 1},

         v:row {
            v:static_text  { title = "  " .. LOC("$$$/167=Caption"), width = LrView.share "_w1"     },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'c1'), value = LrView.bind(CAPTION('p1')), checked_value = 'C',  width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'c2'), value = LrView.bind(CAPTION('p2')), checked_value = 'C',  width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'c3'), value = LrView.bind(CAPTION('p3')), checked_value = 'C',  width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'c4'), value = LrView.bind(CAPTION('p4')), checked_value = 'C',  width = LrView.share "_w2"  },
         },

         v:row {
            v:static_text  { title = "  " .. LOC("$$$/168=Copy Name") , width = LrView.share "_w1"     },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'n1'), value = LrView.bind(CAPTION('p1')), checked_value = 'N', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'n2'), value = LrView.bind(CAPTION('p2')), checked_value = 'N', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'n3'), value = LrView.bind(CAPTION('p3')), checked_value = 'N', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'n4'), value = LrView.bind(CAPTION('p4')), checked_value = 'N', width = LrView.share "_w2"  },
         },

         v:row {
            v:static_text  { title = "  " .. LOC("$$$/169=Headline") , width = LrView.share "_w1"     },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'h1'), value = LrView.bind(CAPTION('p1')), checked_value = 'H', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'h2'), value = LrView.bind(CAPTION('p2')), checked_value = 'H', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'h3'), value = LrView.bind(CAPTION('p3')), checked_value = 'H', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'h4'), value = LrView.bind(CAPTION('p4')), checked_value = 'H', width = LrView.share "_w2"  },
         },

         v:row {
            v:static_text  { title = "  " .. LOC("$$$/170=Job Identifier") , width = LrView.share "_w1"     },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'j1'), value = LrView.bind(CAPTION('p1')), checked_value = 'J', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'j2'), value = LrView.bind(CAPTION('p2')), checked_value = 'J', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'j3'), value = LrView.bind(CAPTION('p3')), checked_value = 'J', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'j4'), value = LrView.bind(CAPTION('p4')), checked_value = 'J', width = LrView.share "_w2"  },
         },

         v:row {
            v:static_text  { title = "  " .. LOC("$$$/171=Title"), width = LrView.share "_w1"     },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 't1'), value = LrView.bind(CAPTION('p1')), checked_value = 'T', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 't2'), value = LrView.bind(CAPTION('p2')), checked_value = 'T', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 't3'), value = LrView.bind(CAPTION('p3')), checked_value = 'T', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 't4'), value = LrView.bind(CAPTION('p4')), checked_value = 'T', width = LrView.share "_w2"  },
         },

         v:row {
            v:static_text  { title = "  " .. LOC("$$$/172=generated filename") , width = LrView.share "_w1" },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'f1'), value = LrView.bind(CAPTION('p1')), checked_value = 'F', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'f2'), value = LrView.bind(CAPTION('p2')), checked_value = 'F', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'f3'), value = LrView.bind(CAPTION('p3')), checked_value = 'F', width = LrView.share "_w2"  },
            v:radio_button { title = "", enabled = LrView.bind(SOURCE_ENABLE 'f4'), value = LrView.bind(CAPTION('p4')), checked_value = 'F', width = LrView.share "_w2"  },
         },
         v:row {
            v:static_text  { title = "  " .. LOC("$$$/173=(unused)"),    width = LrView.share "_w1"     },
            v:radio_button { title = "",                                            value = LrView.bind(CAPTION('p1')), checked_value = 'X', width = LrView.share "_w2"  },
            v:radio_button { title = "",                                            value = LrView.bind(CAPTION('p2')), checked_value = 'X', width = LrView.share "_w2"  },
            v:radio_button { title = "",                                            value = LrView.bind(CAPTION('p3')), checked_value = 'X', width = LrView.share "_w2"  },
            v:radio_button { title = "",                                            value = LrView.bind(CAPTION('p4')), checked_value = 'X', width = LrView.share "_w2"  },
         },
      },

      v:static_text { title = LrView.bind ('_source_priority_note_' .. tag) ,
                      height_in_lines = tonumber(LOC("$$$/160/height=2")),
                      fill_horizontal = 1 }
   }
end

local function blank_to_nil(string)
   if not string or string == "" then
      return nil
   else
      return string
   end
end

-- 'photo' is an LrPhoto object
local function get_one(letter, photo, filename)

   if letter == 'F' then
      -- return filename, sans extension
      return blank_to_nil(filename:gsub('%.[^.]+$', '', 1))
   end

   if letter == 'C' then return blank_to_nil(photo:getFormattedMetadata('caption'))       end
   if letter == 'T' then return blank_to_nil(photo:getFormattedMetadata('title'))         end
   if letter == 'N' then return blank_to_nil(photo:getFormattedMetadata('copyName'))      end
   if letter == 'H' then return blank_to_nil(photo:getFormattedMetadata('headline'))      end
   if letter == 'J' then return blank_to_nil(photo:getFormattedMetadata('jobIdentifier')) end
   return nil
end

function caption_source.get(state, photo, filename, tag)

   -- default to the first set if no tag specified
   if not tag then
      tag = my.sets[1].tag 
   end

   local caption = nil
   if not caption then caption = get_one(state[__CAPTION(tag, 'p1')], photo, filename) end
   if not caption then caption = get_one(state[__CAPTION(tag, 'p2')], photo, filename) end
   if not caption then caption = get_one(state[__CAPTION(tag, 'p3')], photo, filename) end
   if not caption then caption = get_one(state[__CAPTION(tag, 'p4')], photo, filename) end
   return caption
end

return function(options)
          my = options

          --
          -- In the options table passed to us, we expect these keys:
          --
          assert(my.provider)   -- e.g. "Flikr"
          assert(my.sets)

          my.tag2set = {}

          for i, set in ipairs(my.sets) do
             assert(set.label)      -- what the provider calls the title/caption, e.g. "caption"
             assert(set.p1)         -- letter for first-priority source (e.g. 'C' == Caption, as per LongDesc)
             assert(set.p2)         -- letter for 2nd-priority...
             assert(set.p3)         -- letter for 3rd-priority...
             assert(set.p4)         -- letter for 4th-priority...

             if not set.tag then
                set.tag = set.label
             end

             my.tag2set[set.tag] = set
          end

          -- Return a table that exposes this module's public functions
          return caption_source
       end
