--
-- Copyright 2008 Jeffrey Friedl
-- http://regex.info/blog/
--
--

-- system imports
local LrFunctionContext = import 'LrFunctionContext'
local LrBinding         = import 'LrBinding'
local LrDialogs         = import 'LrDialogs'
local LrColor           = import 'LrColor'
local LrPrefs           = import 'LrPrefs'
local LrView            = import 'LrView'

-- local imports
local util              = require 'util'

local my = {}
local interactive_destination = {}


interactive_destination.MagicValue = "@export@"

function interactive_destination.exportPresetFields()

   return {
      --{ key = 'previous_destination', default = '' },
   }

end


function interactive_destination.add_select_element(state, T)
   -- the 'title' must begin with "--" for gallery sorting to work
   table.insert(T, { title = "-- " .. LOC("$$$/137=select at export time (useful for export presets)") .. " --", value = interactive_destination.MagicValue })
end

function interactive_destination.filter_items(old)
   local new = {}

   for i,v in ipairs(old) do
      if v.value ~= interactive_destination.MagicValue then
         table.insert(new, v)
      end
   end

   return new
end


local function query(context, state, count)

   if my.refresh_function and not my.refresh_function(state) then
      LrDialogs.message(LOC("$$$/138=Not logged in to ^1",  my.provider),
                        LOC("$$$/139=You can login via the Export dialog"),
                        "critical")
      return nil
   end

   local v = LrView.osFactory()


   local photos
   if count == 1 then
      photos = LOC("$$$/142=image")
   else
      photos = tostring(count) .. " " .. LOC("$$$/143=images")
   end
   local HEADER = v:static_text { title = LOC("$$$/145=Choose the ^1 for the ^2:", my.text_target, photos) }

   local contents
   local result_fcn

   if my.creategallery_dialog then

      local select_item = "_value_to_use"

      contents = v:view {
         spacing = 5,
         HEADER,
         my.creategallery_dialog(v, state, 'interactive destination', select_item)
      }
      result_fcn = function()
                      return state[select_item]
                   end


   else

      local props = LrBinding.makePropertyTable(context)

      --
      -- Make a copy of state[my.list_key], but the entry with the magic value
      --
      function make_tmp_gallery_list()
         props.tmp_galleries = interactive_destination.filter_items(state[my.list_key])

         if state[my.select_key] == '' or not util.popupmenu_items_has_value(props.tmp_galleries, state[my.select_key]) then
            props.selection = props.tmp_galleries[1].value
            --DebugNote("selection defaults to first")
         else
            props.selection = state[my.select_key]
            --DebugNote("select is " .. tostring(state[my.select_key]))
         end
      end

      make_tmp_gallery_list()

      if #props.tmp_galleries == 0 then
         LrDialogs.message(LOC("$$$/140=No ^1 available", my.text_target),
                           LOC("$$$/141=You must create some before you can export"),
                           "critical")
         return false
      end

      result_fcn = function()
                      return props.selection
                   end

      contents = v:view {
         HEADER,
         v:view {
            bind_to_object = state,

            place = 'overlapping',

            v:static_text {
               visible = LrBinding.keyIsNotNil '_creating_new_gallery',
               title = "working.....",
               text_color = LrColor(1, .2, .1),
            },
            v:popup_menu {
               visible = LrBinding.keyIsNil '_creating_new_gallery',
               save_frame = 'interactive_destination__dialog_settings',
               items = LrView.bind { object = props, key = 'tmp_galleries' },
               value = LrView.bind { object = props, key = 'selection' },
               width_in_chars = 50,
            }
         }
      }

   end

   local UserReply = LrDialogs.presentModalDialog {
      title = LOC("$$$/144=Choose The Destination ^1", my.title_target),
      resizable = true,
      actionVerb = LOC("$$$/130=Continue With Export"),
      cancelVerb = LOC("$$$/131=Cancel Export"),
      contents = contents,
   }

   if UserReply == 'ok' then
      if my.create_gallery then
         my.create_gallery(state)
      end
      return result_fcn()
   else
      return nil
   end
end

function interactive_destination.get(state, count)


   local DEST_GALLERY = state[my.select_key]

   if DEST_GALLERY == interactive_destination.MagicValue then
      LrFunctionContext.callWithContext("custom dialog", function(context)
                                                            DEST_GALLERY = query(context, state, count)
                                                         end)
      if not DEST_GALLERY then
         return nil
      end
   end

   -- Changes we make in processRenderedPhotos to exportContext.propertyTable (AKA "state")
   -- do not necessarily get reflected back to the LR catalog, so we have to be sure to
   -- save them via LrPrefs as well
   --local registeredState = LrPrefs.prefsForPlugin(_PLUGIN.id)
   --registeredState.previous_destination = DEST_GALLERY

   return DEST_GALLERY

end

function interactive_destination.args()
   return my
end


return function(options)
          my = options
          assert(my.provider)
          assert(my.text_target)
          assert(my.title_target)
          assert(my.select_key)
          return interactive_destination
       end