--
-- Copyright 2008 Jeffrey Eric Francis Friedl
-- http://regex.info/blog/
--
-- Some simple table routines.
--

local tableutil = {}

--
-- If the value 'targetvalue' is found in the table 't', returns the first
-- key by which it's found. Otherwise, returns nil.
--
function tableutil.contains(t, targetvalue)
   if targetvalue then
      for key, tablevalue in ipairs(t) do
         if tablevalue == targetvalue then
            return key
         end
      end
   end
   return nil
end

--
-- Given a bunch of tables as arguments, return one table that contains all
-- the key/value pairs. Items with numeric keys are just thrown in in the
-- numeric order. Items with non-numeric keys retain their keys, but may be
-- overwritten by later items from subsequent tables.
--
function tableutil.union(...)
   local union = {}
   for i = 1, select( '#', ... ) do
      for key, value in pairs(select(i, ...)) do
         if type(key) == "number" then
            table.insert(union, value)
         else
            union[key] = value
         end
      end
   end
   return union
end

--
-- Given two tables, append all the stuff in the second table to the first (in place)
-- Returns the first.
--
function tableutil.concatenate(t1, t2)
   for key, value in pairs(t2) do
      if type(key) == "number" then
         table.insert(t1, value)
      else
         t1[key] = value
      end
   end
   return t1
end

--
-- Given a table and a bunch of stuff, inserts all the stuff into the table.
-- Basically, a vararg version of table.insert()
--
-- Returns the table itself
function tableutil.append(t, ...)
   for i = 1, select( '#', ... ) do
      table.insert(t, select(i, ...))
   end
   return t
end

--
-- Returns a simple copy of the table, one level deep. A value that's a table
-- is therefore copied by reference.
--
function tableutil.simplecopy(t)

   local new_t = {}

   for k,v in pairs(t) do
      new_t[k] = v
   end

   return new_t

end

return tableutil

